#include "EC2Manager.h"
#include "myEC2.h"
#include "ConfigureManager.h"
#include "ExceptionManager.h"

//string EC2Manager::EC2SpotInstancePrice = ConfigureManager::appSettings("EC2SpotInstancePrice");
//string EC2Manager::EC2KeyName = ConfigureManager::appSettings("EC2KeyName");
//string EC2Manager::userScriptPath = ConfigureManager::appSettings("userScriptPath");

using namespace std;
using namespace CloudResourceManager;


// utility
string EC2Manager::getSingleResult(const MyEC2& myEC2)
{
    StringPairVector results = myEC2.getResult();
    string res;
    if (results.size() > 0 && results[0].second.find(" 200 OK") != string::npos)
    {
        StringPairVector param = myEC2.getParam();

        if (param.size() == 1)
        {
            StringPairVector::const_iterator it = param.begin();
            res = it->second;
            FILE_LOG(logINFO)<<"[getSingleResult]"<<it->first<<"="<<res;
            //printf("result = %s\n", res.c_str());
            //
        }
    }
    return res;
}

string EC2Manager::ec2_request_spot_instance(string ami, string instanceType)
{
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_request_spot_instance:ami=" + ami + " type:" + instanceType);
    FILE_LOG(logDEBUG)<<"[ec2_request_spot_instance] ami="<<ami<<";instanceType="<<instanceType;
    string EC2SpotInstancePrice = ConfigureManager::appSettings("EC2SpotInstancePrice");
    string EC2KeyName = ConfigureManager::appSettings("EC2KeyName");
    string result;
    if (ami.size() > 0 && instanceType.size() > 0)
    {
        MyEC2 myEC2;
        myEC2.setParam(1, "spotInstanceRequestId");
        myEC2.call(5, "Action", "RequestSpotInstances", "LaunchSpecification.ImageId", ami.c_str(),
            "SpotPrice", EC2SpotInstancePrice.c_str(), "LaunchSpecification.KeyName", EC2KeyName.c_str(),
            "LaunchSpecification.InstanceType", instanceType.c_str());
        result = getSingleResult(myEC2);
        printf("result of request spot instance %s\n", result.c_str());
    }
    return result;
}

bool EC2Manager::ec2_destroy_instance(string instanceId)
{
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_destroy_instance:instanceId=" + instanceId);
    FILE_LOG(logDEBUG)<<"[ec2_destroy_instance] instanceId="<<instanceId;
    if ( instanceId.size() > 0)
    {
        MyEC2 myEC2;
        myEC2.call(2, "Action", "TerminateInstances", "InstanceId.1", instanceId.c_str());
        const StringPairVector& results = myEC2.getResult();
        if (results[0].second.find(" 200 OK") != string::npos)
        {
            FILE_LOG(logINFO)<<"[ec2_destroy_instance] success!";
            return true;
        }
    }
    return false;
}

string EC2Manager::ec2_check_spot_instance_request(string requestId)
{
    string result;
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_check_spot_instance_request:requestId=" + requestId);
    FILE_LOG(logDEBUG)<<"[ec2_check_spot_instance_request] requestId="<<requestId;
    if (requestId.size() > 0)
    {
        MyEC2 myEC2;
        myEC2.setParam(1, "state");
        myEC2.call(2, "Action", "DescribeSpotInstanceRequests", "SpotInstanceRequestId.1", requestId.c_str());
        result = getSingleResult(myEC2);
    }
    return result;
}


string EC2Manager::ec2_request_normal_instance(string ami, string instanceType)
{
    string result;
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_request_normal_instance:ami=" + ami + " instanceType:" + instanceType);
    FILE_LOG(logDEBUG)<<"[ec2_request_normal_instance] ami="<<ami<<";instanceType="<<instanceType;
    string EC2KeyName = ConfigureManager::appSettings("EC2KeyName");
    if (ami.size() > 0 && instanceType.size() > 0)
    {
        try
        {
            MyEC2 myEC2;
            myEC2.setParam(1, "instanceId");
            myEC2.call(4, "Action", "RunInstances", "ImageId", ami.c_str(),
                "KeyName", EC2KeyName.c_str(), "InstanceType", instanceType.c_str());
            result = getSingleResult(myEC2);
        }
        catch (std::runtime_error e)
        {
            FILE_LOG(logERROR)<<e.what();
        }
    }
    return result;
}

string EC2Manager::ec2_check_instance_status(string instanceId)
{
    string result;
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_check_instance_status:instanceId=" + instanceId);
    FILE_LOG(logDEBUG)<<"[ec2_check_instance_status]"<<"instanceId="<<instanceId;
    if (instanceId.size() > 0)
    {
        MyEC2 myEC2;
        myEC2.setParam(1, "name");
        myEC2.call(2, "Action", "DescribeInstances", "InstanceId.1", instanceId.c_str());
        result = getSingleResult(myEC2);
    }
    return result;
}

string EC2Manager::ec2_get_instance_id(string requestId)
{
    string result;
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_check_instance_status:instanceId=" + instanceId);
    FILE_LOG(logDEBUG)<<"[ec2_get_instance_id]"<<"requestId="<<requestId;
    if (requestId.size() > 0)
    {
        MyEC2 myEC2;
        myEC2.setParam(1, "instanceId");
        myEC2.call(2, "Action", "DescribeSpotInstanceRequests", "SpotInstanceRequestId.1", requestId.c_str());
        result = getSingleResult(myEC2);
    }
    return result;
}

string EC2Manager::ec2_get_instance_ip(string instanceId)
{
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_get_instance_ip:instanceId=" + instanceId);
    string result;
    //ConfigLogger.Instance.LogDebug("ec2lib: ec2_check_instance_status:instanceId=" + instanceId);
    FILE_LOG(logDEBUG)<<"[ec2_get_instance_ip]"<<"instanceId="<<instanceId;
    if (instanceId.size() > 0)
    {
        MyEC2 myEC2;
        myEC2.setParam(1, "privateIpAddress");
        myEC2.call(2, "Action", "DescribeInstances", "InstanceId.1", instanceId.c_str());
        result = getSingleResult(myEC2);
    }
    return result;
}


  /*
    class EC2InstanceDescriptor
    {
        public string instanceId;
        public string state;
        public string privateIp;
        public string publicIp;
        public string instanceType;
        public string privateDns;
        public string publicDns;
        public string launchTime;
    }

    class EC2RequestDescriptor
    {
        public string requestId;
        public string instanceId;
        public string state;
    }
    */
